﻿#!python3.4
# phy_startup_test.py
#
# This file is part of python sample package for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2018 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import sys, os
import platform
import time, datetime

from broadway2 import Broadway


Master = 1
Slave = 0

"""
Launch as:

    phy_startup_test m -> startup phy as Master and measure startup time
    phy_startup_test s -> startup phy as Slave and measure startup time

    phy_startup_test <m|s> pollCount default is 5000 times to read register 23

"""

def main(args):
    print('args: {0}'.format(args))
    pollCount = 5000

    if len(args) < 2:
        testMode = Slave    
    elif len(args) >= 2:
        if args[1].upper() == 'M':
            testMode = Master
        elif args[1].upper() == 'S':
            testMode = Slave
        if len(args) > 2:
            pollCount = int(args[2])

    usedAdapter = None

    print('Phy Startup with measurement:')
    print('-'*65)

    # create adapter singleton
    base = Broadway()

    if base.Count < 1:
        print('No adapter found. Exit!')
        sys.exit(-1)

    if base.Count > 0:
        adapters = []

        cnt = 1
        for adapter in base.enumerate():
            adapters.append((cnt, adapter))
            cnt += 1

    if len(adapters) == 1:
        usedAdapter = adapters[0][1]
    else:
        for a in adapters:
            print('{0:2} - {1}/{2}'.format(a[0], a[1].SerialNoBase, a[1].FriendlyName))
        sel = input('Number to use?')
        usedAdapter = adapters[int(sel)-1][1]
        
    print('Adapter: {0}/{1} {2}({3})'.format(usedAdapter.FriendlyName, usedAdapter.PhyType, usedAdapter.SerialNoBase, datetime.datetime.now()))
    dev = base.open_control(usedAdapter.Inst)
    usedAdapter.connect(dev)    
    phy = usedAdapter.get_phy()

    # Test-Sequence starts here
    phy.configEnable = True
    
    if testMode == Master:
        phy.phyMaster = True
    else:
        phy.phyMaster = False

    phy.phyLinkControl = False
    inp = input('TestMode: {0}. Press key to start!'.format(testMode))

    
    regs = []
    regs.append(phy.read_register(23))
    tm1 = datetime.datetime.now()
    phy.phyLinkControl = True   
    
    for j in range(pollCount):
        regs.append(phy.read_register(23))

    tm2 = datetime.datetime.now()
    diffTm = tm2-tm1
    diffPoll = (diffTm.seconds * 1e6 + diffTm.microseconds) / pollCount
    print('Record {0} samples took {1} ({2})'.format(pollCount, tm2-tm1, diffPoll))

    # verify first linkUp
    tm = 0
    tmLinkUp = 0
    linkUpGlitchCnt = 0
    for reg in regs:
        if tmLinkUp == 0:
            if reg & 0x8000:
                tmLinkUp = tm
        else:    # verify after linkUp no glitches
            if (reg & 0x8000) == 0x0000:
                linkUpGlitchCnt += 1

        tm += diffPoll

    print('Link-Up after: {0} us. GlitchCnt: {1}'.format(tmLinkUp, linkUpGlitchCnt))
            
    base.close(dev)


if __name__ == "__main__":
    main(sys.argv)
